############################################
## ANALYSIS OF POPULARITY FROM ASCII MAPS ##
############################################

##############################################################################
## Author: Dr. Claudine Gravel-Miguel                                       ##
##                                                                          ##
## This script uses the path popularity maps created by the Parse scripts.  ##
## It creates buffers around each site, and calculates the mean popularity  ##
## of paths within those buffers.                                           ##
##                                                                          ##
## The results are printed in the console rather then saved to a file.      ##
##                                                                          ##
##############################################################################

# These packages need to be installed for the script to work.
library(raster)
library(ggplot2)
library(reshape)
library(dplyr)
library(rgeos)
library(rgdal)

# Clears the environment to avoid errors from old files
rm(list=ls())

# This function reads the geographical maps and create the necessary buffers
calculate.buffer.popularity <- function(){
  
  # Browse to the main DEM
  DEM <- raster(file.choose(), sep=",")
  
  # Browse to the shapefile of the sites' locations (vector), for which to get popularity values.
  # ** Make sure it is in the same projection as the DEM.
  shp <- readOGR(file.choose())
  
  # Browse to the raster map to analyse
  map <- raster(file.choose(), sep=",")
  
  # Browse to the map to use as a mask for possible paths ("Exploration" for site-pair paths, "DEM" for FETE paths)
  buffer <- raster(file.choose(), sep=",")
  
  # Simply the buffer to use as a mask
  buffer[buffer > 0] <- 0
  buffer[buffer < 0] <- NA
  
  # Merge map and buffer so that NA cells in map that are within the buffer are set to 0.
  map2 <- resample(map, buffer, "bilinear")
  full <- merge(map2,buffer)
  
  # Transform the raster into a matrix of coordinates and values
  whole <- rasterToPoints(full)

  # Iterate over the different site coordinates to obtain the mean popularity values of the cells in a 1km buffer
  for(i in 1:nrow(shp)) {
    
    # Transform the site into a raster point
    site.ras <- rasterize(shp[i,], map, field = 1)
    
    # Create a polygon of 1km radius around the site raster
    pol <- rasterToPolygons(site.ras)
    pol.buf <- buffer(pol, width=1000) 
    
    # Use the buffer polygon to clip the Popularity map uploaded 
    mask.ras <- mask(map,pol.buf)
    site.list <- rasterToPoints(mask.ras)
    
    # If the site's buffer falls onto at least 2 cells walked on by an agent, 
    # this calculates the mean popularity of the cells within the buffer and uses a T-test to evaluate if these are significantly more popular
    # than all the cells walked on.
    
    if (nrow(site.list) > 1){
      # ** Here, "str_3" represents the column that holds the name of the site. It can be changed for other shapefiles.
      print(shp[i,]$str_3)     
      print(t.test(site.list[,3],whole[,3], alternative = "greater"))
      
    }else{                      
      # If the site's buffer is outside of the paths, nothing happens.
      print(s)
      print("NA")
    }
  }
}

# This calls the procedure. All temporary variables will not be recorded in the Global Environment, to save space and speed.
# It will prompt a browsing window for the following files:

# 1st: DEM
# 2nd: Shapefile of sites' locations
# 3rd: Raster of paths from which to get the popularity
# 4th: Raster of buffer used as mask around the paths (Exploration map, or DEM)

calculate.buffer.popularity()
